// ZDriver_Be.h
// Andrew Green, ag@electricmagic.com

#ifndef __ZDriver_Be__
#define __ZDriver_Be__ 1

#include <drivers/Drivers.h> // For device_hooks

#ifndef nil
	#define nil (0)
#endif // nil

class ZDriver_Be_Device;
class ZDriver_Be_Card;
class ZDriver_Be_DeviceInstance;
class ZDriver_Be_Driver
	{
public:
	static status_t sInitHardware();
	static status_t sInitDriver();

	static status_t sOpen(const char* inName, uint32 inFlags, void** outCookie);
	static status_t sClose(void* inCookie);
	static status_t sFree(void* inCookie);
	static status_t sControl(void* inCookie, uint32 inOperation, void* inOutData, size_t inLen);
	static status_t sRead(void* inCookie, off_t inPos, void* outData, size_t* inOutLen);
	static status_t sWrite(void* inCookie, off_t inPos, const void* inData, size_t* inOutLen);	

public:
	ZDriver_Be_Driver();
	virtual ~ZDriver_Be_Driver();
	
	virtual bool Initialize();
	virtual void Finalize();

	const char** PublishDeviceNames();

	ZDriver_Be_Device* FindDeviceByName(const char* inName);

	bool AddCard(ZDriver_Be_Card* inCard);
	
private:
	static const size_t kMaxCards = 8;
	ZDriver_Be_Card* fCards[kMaxCards];
	size_t fCountCards;
	const char** fDeviceNamesCached;

	friend void uninit_driver();
	friend const char** publish_devices();
	friend device_hooks* find_device(const char* inName);

	static ZDriver_Be_Driver* sDriver;
	static device_hooks sDeviceHooks;
	};

// ==================================================

class ZDriver_Be_Card
	{
public:
	ZDriver_Be_Card();
	virtual ~ZDriver_Be_Card();

	virtual bool Initialize();
	virtual void Finalize();

	size_t CountNames();
	const char* GetNameByIndex(size_t inNameIndex);
	ZDriver_Be_Device* FindDeviceByName(const char* inName);

	bool AddDevice(ZDriver_Be_Device* inDevice);
	
protected:
	static const size_t kMaxDevices = 8;

	ZDriver_Be_Device* fDevices[kMaxDevices];
	size_t fCountDevices;
	};

// ==================================================

class ZDriver_Be_Device
	{
public:
	ZDriver_Be_Device();
	virtual ~ZDriver_Be_Device();

	virtual bool Initialize();
	virtual void Finalize();

	virtual size_t CountNames() = 0;
	virtual const char* GetNameByIndex(size_t inNameIndex) = 0;

	virtual status_t Open(const char* inName, uint32 inFlags, ZDriver_Be_DeviceInstance*& outDeviceInstance) = 0;
	};

// ==================================================

class ZDriver_Be_DeviceInstance
	{
public:
	ZDriver_Be_DeviceInstance();
	virtual ~ZDriver_Be_DeviceInstance();

	virtual status_t Close() = 0;
	virtual status_t Free() = 0;
	virtual status_t Control(uint32 inOperation, void* inOutData, size_t inLen) = 0;
	virtual status_t Read(off_t inPos, void* outData, size_t& inOutLen) = 0;
	virtual status_t Write(off_t inPos, const void* inData, size_t& inOutLen) = 0;
	};

// ==================================================
// Stuff ripped from ZDebug.h
#ifndef ZCONFIG_Debug
	#define ZCONFIG_Debug 2
#endif // ZCONFIG_Debug

enum ZDebug_Action { eDebugAction_Continue, eDebugAction_Throw, eDebugAction_Stop };

// These are the internal methods that get called to set up and display a debugging message.
void ZDebug_DisplayMessage(int level, ZDebug_Action action, const char* filename, int line, const char* assertionMessage, const char* userMessage);
// We use operator overloading on ZDebug_FormatMessage to help the macros out.
const char* ZDebug_FormatMessage(const char* message, ...);
inline const char* ZDebug_FormatMessage() { return 0; }


// The actual macros that get used in source code. The funkiness of these macros (should) generate minimal inline code, so that
// an assertion that's active at level 2 will generate *no code* at level 1 or 0. This does rely on the compiler optimizing
// away statements/expressions with no effect.
#define ZDebugLogf(a, b) ((a)<=ZCONFIG_Debug ? ZDebug_DisplayMessage(a, eDebugAction_Continue, __FILE__, __LINE__, 0, ZDebug_FormatMessage b): (void)0)
#define ZDebugThrowf(a, b) ((a)<=ZCONFIG_Debug ? ZDebug_DisplayMessage(a, eDebugAction_Throw, __FILE__, __LINE__, 0, ZDebug_FormatMessage b): (void)0)
#define ZDebugStopf(a, b) ((a)<=ZCONFIG_Debug ? ZDebug_DisplayMessage(a, eDebugAction_Stop, __FILE__, __LINE__, 0, ZDebug_FormatMessage b): (void)0)
#define ZDebugLog(a) ((a)<=ZCONFIG_Debug ? ZDebug_DisplayMessage(a, eDebugAction_Continue, __FILE__, __LINE__, 0, 0): (void)0)
#define ZDebugThrow(a) ((a)<=ZCONFIG_Debug ? ZDebug_DisplayMessage(a, eDebugAction_Throw, __FILE__, __LINE__, 0, 0): (void)0)
#define ZDebugStop(a) ((a)<=ZCONFIG_Debug ? ZDebug_DisplayMessage(a, eDebugAction_Stop, __FILE__, __LINE__, 0, 0): (void)0)

#define ZAssertLogf(a, b, c) ((a)<=ZCONFIG_Debug && !(b) ? ZDebug_DisplayMessage(a, eDebugAction_Continue, __FILE__, __LINE__, #b, ZDebug_FormatMessage c), false: true)
#define ZAssertThrowf(a, b, c) ((a)<=ZCONFIG_Debug && !(b) ? ZDebug_DisplayMessage(a, eDebugAction_Throw, __FILE__, __LINE__, #b, ZDebug_FormatMessage c): (void)0)
#define ZAssertStopf(a, b, c) ((a)<=ZCONFIG_Debug && !(b) ? ZDebug_DisplayMessage(a, eDebugAction_Stop, __FILE__, __LINE__, #b, ZDebug_FormatMessage c), false: true)
#define ZAssertLog(a, b) ((a)<=ZCONFIG_Debug && !(b) ? ZDebug_DisplayMessage(a, eDebugAction_Continue, __FILE__, __LINE__, #b, nil), false: true)
#define ZAssertThrow(a, b) ((a)<=ZCONFIG_Debug && !(b) ? ZDebug_DisplayMessage(a, eDebugAction_Throw, __FILE__, __LINE__, #b, nil): (void)0)
#define ZAssertStop(a, b) ((a)<=ZCONFIG_Debug && !(b) ? ZDebug_DisplayMessage(a, eDebugAction_Stop, __FILE__, __LINE__, #b, nil), false: true)


#define ZUnimplemented() ZDebugStopf(0, ("Unimplemented routine"))

// Legacy macros used all over the place.
#define ZAssert(a) ZAssertStop(1, a)
#define ZDebugString(a) ZDebugStopf(1, (a))

#endif // __ZDriver_Be__